using System.Net;
using Microsoft.EntityFrameworkCore;
using Microsoft.EntityFrameworkCore.Metadata.Builders;

namespace Authinator.Backend.Database.Tables;

public class Network {
	public int       Id     { get; }
	public string    Name   { get; set; } = null!;
	public IPAddress Subnet { get; set; } = null!;

	public class Configuration : IEntityTypeConfiguration<Network> {
		public void Configure(EntityTypeBuilder<Network> group) {
			group.ToTable("Networks");
			group.HasKey(p => p.Id);
			group.Property(b => b.Name).HasColumnName("Name").IsRequired();
			group.Property(b => b.Name).HasColumnName("Subnet").IsRequired();
		}
	}

	private bool Equals(Network other) => Id == other.Id;

	public override bool Equals(object? obj) {
		if (ReferenceEquals(null, obj))
			return false;
		if (ReferenceEquals(this, obj))
			return true;
		return obj.GetType() == GetType() && Equals((Network)obj);
	}

	public override int GetHashCode() => Id;

	public static bool operator ==(Network? left, Network? right) => Equals(left, right);

	public static bool operator !=(Network? left, Network? right) => !Equals(left, right);
}
